/*
 * File: app/controller/Manager/State.js
 *
 * This file was generated by Sencha Architect
 * http://www.sencha.com/products/architect/
 *
 * This file requires use of the Ext JS 4.2.x library, under independent license.
 * License of Sencha Architect does not include license for Ext JS 4.2.x. For more
 * details see http://www.sencha.com/license or contact license@sencha.com.
 *
 * This file will be auto-generated each and everytime you save your project.
 *
 * Do NOT hand edit this file.
 */

/*
	Manager state is a class for field validation and update. Here are the functions required to validate each field. Within the validations, the returns of the functions are for the view, which will wait:

	- Boolean: Usually true, this is when validation has passed.

	- String: Used for when validation fails, the string will be used as field error tooltip

	It's strange multiple returns, but that's how Extjs works with the validators. For convenience reasons it was implemented like this.

	The validation functions also execute the updateState of the model as required.

	NOTE: To unify the validation returns, one idea would be to create a type of validation result.
*/
Ext.define('installwiz.controller.Manager.State', {
	extend: 'Ext.app.Controller',

	refs: [
		{
			ref: 'EulaCheckbox',
			selector: '#mainView #container #body #userForm #eulaContainer #eulaCheckbox'
		},
		{
			ref: 'EulaPopupCheckbox',
			selector: '#mainView #eulaPopup #eulapopupaccept #accept_checkbox'
		}
	],

	onWelcomeMsgBeforeRender: function(component, eOpts) {
		Ext.Ajax.request({
			method: "GET",
			url: "/opmon/seagull/www/index.php/startwizard/action/get_version/",
			success: this.successVersionHandler__.bind(this),
			failure: this.errorVersionHandler__.bind(this)
		});
	},

	/*
		@public
		@constructor
	*/
	init: function(application) {
				var controller = installwiz.app.getController("Manager.Intention");
				installwiz.model.Form.Observer.subscribe(controller.changeButtonStartStatus.bind(controller));

		this.control({
			"#description": {
				beforerender: this.onWelcomeMsgBeforeRender
			}
		});
	},

	/*
		@public
		Validate if a string is an email
		@param {x} <String> String value to validate
		@return <String> Represent the message fail || <Bool> Represent validate status
	*/
	validateEmail: function(component, x) {
		if ( installwiz.Support.isNotEmail(x)
			|| installwiz.Support.isEmpty(x)
		   )
		{
			this.insertFormIcon(component, false);
			installwiz.model.Form.Observer.updateState("email", false);
			this.updateEmail__(null);
			return Ext.locale.translate('Must be a valid email.');
		}

		this.insertFormIcon(component, true);
		installwiz.model.Form.Observer.updateState("email", true);
		this.updateEmail__(x);
		return true;
	},

	/*
		@public
		Validate if a string is an password
		@param {x} <String> String value to validate
		@return <String> Represent the message fail || <Bool> Represent validate status
	*/
	validatePassword: function(component, x) {
		if (installwiz.Support.isEmpty(x)) {
			this.insertFormIcon(component, false);
			installwiz.model.Form.Observer.updateState("password", false);
			this.updatePassword__(null);
			return Ext.locale.translate('This field is mandatory');
		}

		this.insertFormIcon(component, true);
		installwiz.model.Form.Observer.updateState("password", true);
		this.validateRepassword(Ext.ComponentQuery.query("#mainView #body #userForm #repassword")[0], installwiz.app.getController("Manager.Intention").getRepasswordValue());
		return true;
	},

	/*
		@public
		Validate if a string is an repassword
		@param {x} <String> String value to validate
		@return <String> Represent the message fail || <Bool> Represent validate status
	*/
	validateRepassword: function(component, x) {
		if (installwiz.app.getController("Manager.Intention").getPasswordValue() !== x) {
			this.insertFormIcon(component, false);
			installwiz.model.Form.Observer.updateState("repassword", false);
			this.updatePassword__(null);
			return Ext.locale.translate('Passwords does not match');
		}

		this.insertFormIcon(component, true);
		installwiz.model.Form.Observer.updateState("repassword", true);
		this.updatePassword__(x);
		return true;
	},

	/*
		@public
		Validate if a string is an valid token
		@param {x} <String> String value to validate
		@return <String> Represent the message fail || <Bool> Represent validate status
	*/
	validateToken: function(component, x) {
		var validate = false;
		var me = this;
		installwiz.model.Form.Observer.updateState("token", false);

		if (x.length < 24) {
			this.insertFormIcon(component, false);
			return Ext.locale.translate('Corrupted license');
		}

		Ext.Ajax.request({
			async: false,
			method: "POST",
			url: "/opmon/seagull/www/index.php/startwizard/action/validate_activation_key/",
			params: { data: x },
			success: function(res) {
				var result = Ext.decode(res.responseText);
				if (result && result.success && result.success === true) {
					me.insertFormIcon(component, true);
					installwiz.model.Form.Observer.updateState("token", true);
					validate = true;
					component.addCls("success-input");
				}
				else if (result && result.message && typeof result.message === "string") {
					me.insertFormIcon(component, false);
					installwiz.model.Form.Observer.updateState("token", false);
					validate = Ext.locale.translate(result.message);
					component.removeCls("success-input");
				}
				else {
					me.insertFormIcon(component, false);
					installwiz.model.Form.Observer.updateState("token", false);
					validate = Ext.locale.translate('Corrupted license');
					component.removeCls("success-input");
				}
			},
			failure: function() {
				me.insertFormIcon(component, false);
				installwiz.model.Form.Observer.updateState("token", false);
				validate = Ext.locale.translate('Corrupted license');
				component.removeCls("success-input");
			}
		});

		return validate;
	},

	/*
		@private
		Updates the password in the model
		@param {x} <String> The password
	*/
	updatePassword__: function(x) {
		installwiz.model.Configurations.setField("pass", x);
	},

	/*
		@private
		Updates the email in the model
		@param {x} <String> The email
	*/
	updateEmail__: function(x) {
		installwiz.model.Configurations.setField("email", x);
	},

	/*
		@public
		Toggle check eula term
	*/
	toggleEulaTerm: function() {
		var checkbox = this.getEulaCheckbox();
		checkbox.setValue(!checkbox.checked);
		this.changeEulaPopupCheckbox__(checkbox.checked);
		return this;
	},

	/*
		@public
		Change check eula term
		@param {value} <Bool> Boolean represent the value to set
	*/
	changeEulaTerm: function(value) {
		this.getEulaCheckbox().setValue(!!value);
		this.changeEulaPopupCheckbox__(!!value);
		return this;
	},

	/*
		@private
		Change check eula popup term. This method is private because the class of Manager.State should be concerned with leaving the two checkboxes with the same value
		@param {value} <Bool> Boolean represent the value to set
	*/
	changeEulaPopupCheckbox__: function(value) {
		this.getEulaPopupCheckbox().setValue(value);
		return this;
	},

	/*
		@private
		When get version, replace
	*/
	successVersionHandler__: function(response) {
		var version =
			installwiz.Maybe.withDefault
				( ""
				, installwiz.Maybe.of(response.responseText)
					 .map(function(x) { return x.replace(/"/g, ''); })
				);

		this.version = version;
		installwiz.app.getController("Translate").translateWelcomeMsg(version);
	},

	/*
		@private
		When get version, replace
	*/
	errorVersionHandler__: function(response) {
		installwiz.app.getController("Translate").translateWelcomeMsg("");
	},

	/*
		@public
		Get opmon version
		@return <String>
	*/
	getVersion: function() {
		return (this.version || "Undefined");
	},

	/*
		@public
		Set opmon version
		@param {value} <String> Opmon version
		@return <this>
	*/
	setVersion: function(value) {
		this.version = value;
		return this;
	},

	/*
		@public
		Set opmon version
		@param {value} <String> Opmon version
		@return <this>
	*/
	insertFormIcon: function(component, status) {
		if (component.notFirstTime) {
			if (status) {
				component.triggerEl.elements[0].removeCls('form-none').removeCls('form-cross').addCls('form-tick');
			}
			else {
				component.triggerEl.elements[0].removeCls('form-none').removeCls('form-tick').addCls('form-cross');
			}
			return this;
		}
		component.notFirstTime = true;
		return this;
	}

});
