(function () {
	
	"use strict";
	
	/*
		Maybe is a set consisting of two directions, Just a value and Nothing.
		Represents the existence or not of a value.
	*/
	
	Ext.define("installwiz.Maybe", function() {
		var Nothing = {
			x: "Nothing",
			map: function(f) { return Nothing; },
			chain: function(f) { return Nothing; },
			ap: function(other) { return Nothing; },
			fold: function(f, g) { return f(); }
		};
		
		var Just = function(x) {
			return {
				x: x,
				map: function(f) { return validate(x) ? Just(f(x)) : Nothing; },
				chain: function(f) { return f(x); },
				ap: function(other) { return other.map(x); },
				fold: function(f, g) { return g(x); }
			};
		};
		
		// validate : a -> Bool
		var validate = function(x) {
			return (x !== null) && (x !== undefined);
		};
		
		// of : a -> Maybe a
		var of = function(x) {
			return (validate (x)) ? Just(x) : Nothing;
		};
		
		// widhtDefault : b -> Maybe a -> a | b
		var withDefault = function(x, ma) {
			return ma.fold(installwiz.Support.always(x), installwiz.Support.identity);
		};
		
		// isNothing : Maybe a -> Bool
		var isNothing = function(ma) {
			return ma.fold
				( installwiz.Support.always(true)
				, installwiz.Support.always(false)
				);
		};
		
		// testEmpty : a -> Maybe a
		var testEmpty = function(x) {
			return installwiz.Support.compose
				([ installwiz.Support.map(installwiz.Support.always(x))
				 , installwiz.Support.chain(installwiz.Support.boolToMaybe)
				 , installwiz.Support.map(installwiz.Support.isNotEmpty)
				 , of
				 ]) (x);
		};
		
		return {
			singleton: true,
			Nothing: Nothing,
			Just: Just,
			of: of,
			withDefault: withDefault,
			isNothing: isNothing,
			testEmpty: testEmpty
		};
	});
	
}) ();
