(function () {
	
	"use strict";
	
	/*
	*/
	
	Ext.define("installwiz.Support", function() {
		return {
			singleton: true,
			
			// compose : [(a -> b)] -> a -> b
			compose: function(xs) {
				return function(init) {
					return xs.reduceRight
							( function(value, f) { return f(value); }
							, init
							);
				};
			},
			
			// pipe : [(a -> b)] -> a -> b
			pipe: function(xs) {
				return function(init) {
					return xs.reduce
							( function(value, f) { return f(value); }
							, init
							);
				};
			},
			
			// isEmpty : a -> Bool
			isEmpty: function(x) {
				return (x === "") || (x.length && x.length === 0);
			},
			
			// isNotEmpty : a -> Bool
			isNotEmpty: function(x) {
				return (installwiz.Support.isEmpty(x) === false);
			},
			
			// isEmail : String -> Bool
			isEmail: function(x) {
				return /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.test(x);
			},
			
			// isNotEmail : String -> Bool
			isNotEmail: function(x) {
				return installwiz.Support.isEmail(x) === false;
			},
			
			// boolToMaybe : Bool -> Maybe Bool
			boolToMaybe: function(flag) {
				return (flag === true)
					? installwiz.Maybe.Just(true)
					: installwiz.Maybe.Nothing;
			},
			
			// always : a -> _ -> a
			always: function(x) {
				return function() { return x; };
			},
			
			// identity : a -> a
			identity: function(x) {
				return x;
			},
			
			map: function(f) {
				return function(ma) {
					return ma.map(f);
				};
			},
			
			chain: function(f) {
				return function(ma) {
					return ma.chain(f);
				};
			},
			
			// trace : String -> a -> a
			trace: function(tag) {
				return function(x) {
					console.log("Trace " + tag + ": ", x);
					return x;
				};
			},
			
			// objectToArray : Object a -> Array a
			objectToArray: function(o) {
				return Object.keys(o).map(function(k) { return o[k]; });
			},
			
			// allReducer : [Bool] -> Bool
			allReducer: function(xs) {
				return xs.reduce
						( function(result, x) { return (result && x); }
						, true
						);
			},
			
			// anyReducer : [Bool] -> Bool
			anyReducer: function(xs) {
				return xs.reduce
						( function(result, x) { return (result || x); }
						, false
						);
			},
			
			// prop : String -> {a} -> a
			prop: function(key) {
				return function(x) { return installwiz.Maybe.of(x[key]); };
			},
			
			// createObjectWith : String -> a -> {a}
			createObjectWith: function(key, x) {
				var fresh = {};
				fresh[key] = x;
				return fresh;
			},
			
			// head : [a] -> Maybe a
			head: function(xs) {
				return installwiz.Maybe.of(xs[0]);
			}
		};
	});
	
}) ();
